{smcl}
{* 21mar2012}{...}
{cmd:help qenvnormal} 
{hline}

{title:Generate quantile envelopes for normal quantile-quantile plot}


{title:Syntax}

{p 8 18 2}
{cmd:qenvnormal} 
{it:varname} 
{ifin} 
{cmd:,}
{cmdab:gen:erate(}{it:newvar1 newvar2}{cmd:)} 
[
{cmd:reps(}{it:#}{cmd:)} 
{cmdab:l:evel(}{it:#}{cmd:)} 
]


{title:Description}

{pstd}
{cmd:qenvnormal} generates two numeric variables for later plotting on a
quantile-quantile plot that define an envelope of low and high
expectations for each quantile given repeated sampling from a normal
distribution with the same number of values, mean and standard deviation
as the data specified. The program is a helper program designed for use
with {help qplot}, which must be installed separately. 
Type {stata findit qplot} to get references and code sources. 

{pstd} 
When plotted, the envelopes are to be considered indicative, and not as
implying a formal decision. In particular, the envelopes underline which
quantiles are least reliable.  With some experience, they should guide
later analysis. 


{title:Remarks} 

{pstd}
The precise recipe is as follows. 

{pstd}
1. Calculate number of values, mean and standard deviation. 

{pstd}
2. Take {cmd:reps()} samples of the same size from a normal distribution
with the same mean and standard deviation. 

{pstd}
3. Order each sample from smallest to largest. 

{pstd}
4. For each rank, calculate reference intervals containing
{cmd:level()}% of the sample quantiles. For example, with the default of
95%, the interval ranges from the 2.5% quantile to the 97.5% quantile,
the quantiles being calculated using a procedure similar to that used
for {help pctile}.  

{pstd}
5. The generated variables contain values defining the reference
intervals.  

{pstd} 
In principle, the envelope variables could also be plotted in
conjunction with {help qnorm}, but that would require more work. 


{title:Options}

{phang}
{cmd:generate()} specifies the names of two new variables to be
generated.  This is a required option.  

{phang} 
{cmd:reps()} specifies the number of samples to be taken. The default is
100.  This is likely to be adequate for informal exploration.  

{phang} 
{cmd:level()} specifies the percent of sample values to be included
within the envelope for each rank.  {cmd:level(100)} is allowed and
interpreted as the entire range. 


{title:Examples} 

{phang}{cmd:. sysuse auto, clear}{p_end}
{phang}{cmd:. qenvnormal weight, gen(lower upper)}{p_end}
{phang}{cmd:. qplot weight lower upper, ms(oh none ..) c(. l l) lc(gs10 ..) legend(off) ytitle("Weight, lb")}{p_end}
{phang}{cmd:. su weight}{p_end}
{phang}{cmd:. qplot weight lower upper, ms(oh none ..) c(. l l) lc(gs10 ..) legend(off) ytitle("Weight, lb") trscale(`r(mean)' + `r(sd)' * invnormal(@)) xtitle(Normal quantiles)}


{title:Author} 

{pstd}Nicholas J. Cox, Durham University{break} 
      n.j.cox@durham.ac.uk 


{title:Also see}

{psee}
Manual:  {bf:[R] diagnostic plots}

